/*
 *  SPDX-FileCopyrightText: 2015 Andreas Cord-Landwehr <cordlandwehr@kde.org>
 *
 *  SPDX-License-Identifier: LGPL-2.1-only OR LGPL-3.0-only OR LicenseRef-KDE-Accepted-LGPL
 */

#ifndef LOGGING_H
#define LOGGING_H

#include <QLoggingCategory>

Q_DECLARE_LOGGING_CATEGORY(GRAPHTHEORY_FILEFORMAT)
Q_DECLARE_LOGGING_CATEGORY(GRAPHTHEORY_GENERAL)
Q_DECLARE_LOGGING_CATEGORY(GRAPHTHEORY_KERNEL)

#endif
